/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	Class performing decoding of JPEG2000 codestream data on CPU with Kakadu.

***********************************************************************************/

#ifndef HOSTDECODER_H
#define HOSTDECODER_H

//Project includes
//#include "Thread.h"
#include "PacketBuffer2CodeStream.h"
#include <cudaLib.h>

// System includes
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <fstream>
// Kakadu core includes
#include "kdu_compressed.h"
#include "kdu_elementary.h"
#include "kdu_messaging.h"
#include "kdu_params.h"
#include "kdu_sample_processing.h"
//Kakadu application level includes
#include "kdu_stripe_decompressor.h"
#include "kduCompressedMemorySource.h"

using namespace std;

#define imgWidth 1280
#define imgHeight 720

//Max bytes in decoded frame: 7500000(1.5Gbit/s @ 25fps)

class HostDecoder:public Thread{

public:
	HostDecoder(char** codestreamBuffer[], int** bytesInBuffer);
	void RunPerformanceTest();
	void Execute();
	void RunCorrectnessTest();
	void SetDbufHeight(int newDbufHeight);

	unsigned char *hostOutput0[3];
	unsigned char *hostOutput1[3];

private:
	bool kduDecodeFrame(int bufferNumber);
	void SaveKduOutput();
	void DeinterleaveVisual(unsigned char * hostLocal[3], int scale, int level);

	kduCompressedMemorySource codestreamSource;
	kdu_codestream codestream;
	kdu_stripe_decompressor decompressor;
	kdu_thread_env env, *env_ref;

	char *jpcBuffer[CODESTREAM_BUFFER_LENGTH];
	int *bytesInJpcBuffer;
	int stripeHeights[3];
	int sampleGaps[3];
	int envDbufHeight;

	HANDLE hostOutput0Mutex, hostOutput1Mutex;
	HANDLE codestreamBuff0, codestreamBuff1;
	HANDLE codestreamBufferSemaphore;
	ofstream debugLog;

};

class kdu_stream_message : public kdu_message {
public: // Member classes
	kdu_stream_message(std::ostream *stream)
	{ this->stream = stream; }
	void put_text(const char *string)
	{ (*stream) << string; }
	void flush(bool end_of_message=false)
	{ stream->flush(); }
private: // Data
	std::ostream *stream;
};

static kdu_stream_message cout_message(&std::cout);
static kdu_stream_message cerr_message(&std::cerr);
static kdu_message_formatter pretty_cout(&cout_message);
static kdu_message_formatter pretty_cerr(&cerr_message);


//Simple PGM/PPM file writer.  The `buffer' array contains interleaved
//image samples (there are 1 or 3 components) in unsigned bytes,
//appearing in scan-line order.
static void writeImage(char *fname, kdu_byte *buffer, int num_components, int height, int width, bool is8bit, bool createHeader)
{
	std::ofstream out(fname,std::ios::out|std::ios::binary);
	if (!out)
	{ kdu_error e;
	e << "Unable to open output image file, \"" << fname << "\"."; }
	if ((num_components == 1) & createHeader){
		//writing gray map with 8bpp
		if(is8bit){
			out << "P5\n" << width << " " << height << "\n255\n";
			out.write((char *) buffer, width*height);
		}
		//writing gray map with 16bpp
		else{
			out << "P5\n" << width << "\n" << height << "\n65535\n";
			out.write((char *) buffer, 2*width*height);

		}
	}
	//writing color map with 8bpp
	else if ((num_components == 3) & createHeader){
		out << "P6\n" << width << " " << height << "\n255\n";
		out.write((char *) buffer, 3*width*height);
	}
	//writing gray raw with 8bpp
	else if ((num_components == 1) & !createHeader & is8bit){
		out.write((char *) buffer, width*height);
	}
}

#endif