/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	Class reconstructing codestream from several udp packets.

***********************************************************************************/

#include "PacketBuffer2Codestream.h"
#include "CudaLib.h"

//Constructor.
//Parameters:
//packetBuffer [in] - Pointer to pointer to the packet payload buffer.
//bytesInBuffer [in] - Pointer to pointer to the number of bytes in the packet payload buffer.
//
//Return:
PacketBuffer2Codestream::PacketBuffer2Codestream(char** packetBuffer[],int** bytesInBuffer, enum INPUT_VIDEO_FORMAT videoFormat){

	inputVideoFormat=videoFormat;
	bool constructorSucceeded = true;
	//Initializing debugLog
	remove("codestream_debug_log.txt");
	debugLog.open("codestream_debug_log.txt",ios_base::out);

	//Allocating memory for codestreamBuffer
	for(int i=0; i<CODESTREAM_BUFFER_LENGTH;i++){
		jpcBuffer[i]=new (nothrow) char[CODESTREAM_BUFFER_WIDTH];
		//jpcBuffer[i]=(char*)AllocateMemHost(CODESTREAM_BUFFER_WIDTH);
		bytesInJpcBuffer[i]=0;
		if (jpcBuffer[i] == 0) {
			constructorSucceeded=false;
			debugLog<<"Allocating memory for jpcBuffer failed in iteration " << i <<endl;
			break;
		}
	}

	chromaTempBuffer = new (nothrow) char[CODESTREAM_BUFFER_WIDTH];
	//chromaTempBuffer = (char*)AllocateMemHost(CODESTREAM_BUFFER_WIDTH);
	if (chromaTempBuffer == 0) {
		constructorSucceeded=false;
		debugLog<<"Allocating memory for chromaTempBuffer failed." <<endl;
	}

	//Copying pointers to udpPayloadBuffer and bytesInBuffer.
	for(int i=0;i<PAYLOAD_BUFFER_LENGTH;i++){
		updPayloadBuffer[i]=*(packetBuffer[i]);
	}
	bytesInUdpPayload=*bytesInBuffer;

	//Opening mutexes and semaphore to udpPayloadBuffer
	bufferMutexPartition0 = OpenMutex(STANDARD_RIGHTS_ALL, false,(LPCTSTR)"bufferMutexPartition0");
	if(bufferMutexPartition0==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed opening bufferMutexPartition0 with error code: 0x"<<hex<<error<<"."<<endl;
		exit(-1);
	}
	bufferMutexPartition1 = OpenMutex(STANDARD_RIGHTS_ALL, false,(LPCTSTR)"bufferMutexPartition1");
	if(bufferMutexPartition1==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed opening bufferMutexPartition1 with error code: 0x"<<hex<<error<<"."<<endl;
		exit(-1);
	}
	bufferMutexPartition2 = OpenMutex(STANDARD_RIGHTS_ALL, false,(LPCTSTR)"bufferMutexPartition2");
	if(bufferMutexPartition2==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed opening bufferMutexPartition2 with error code: 0x"<<hex<<error<<"."<<endl;
		exit(-1);
	}
	bufferMutexPartition3 = OpenMutex(STANDARD_RIGHTS_ALL, false,(LPCTSTR)"bufferMutexPartition3");
	if(bufferMutexPartition3==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed opening bufferMutexPartition3 with error code "<<error<<"."<<endl;
		exit(-1);
	}
	packetBufferSemaphore = OpenSemaphore(STANDARD_RIGHTS_ALL, false,(LPCTSTR)"packetBufferSemaphore");
	if(packetBufferSemaphore==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed opening bufferSemaphore with error code: 0x"<<hex<<error<<"."<<endl;
		exit(-1);
	}

	//Creating codestreamBuffer mutexes and semaphore
	codestreamBufferSemaphore = CreateSemaphore(NULL, 0,4,(LPCTSTR)"codestreamBufferSemaphore");
	if(codestreamBufferSemaphore==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating codestreamBufferSemaphore with error code 0x"<<hex<<error<<"."<<endl;
	}
	codestreamBuff0 = CreateMutex(NULL, false,(LPCTSTR)"codestreamBuff0");
	if(codestreamBuff0==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating codestreamBuff0 with error code 0x"<<hex<<error<<"."<<endl;
	}
	codestreamBuff1 = CreateMutex(NULL, false,(LPCTSTR)"codestreamBuff1");
	if(codestreamBuff1==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating codestreamBuff1 with error code 0x"<<hex<<error<<"."<<endl;
	}
}//end PacketBuffer2CodeStream()

//Returning JPEG2000 codestream buffer pointer for use in decoder.
//Parameters:
//codestreamBuffer [out] - Pointer to pointer to the codestream buffer.
//bytesInBuffer [out] - Pointer to pointer to the codestream buffer size.
//Return: void
void PacketBuffer2Codestream::GetCodestreamBuffer(char** codestreamBuffer[], int** bytesInBuffer){

	for(int i=0;i<CODESTREAM_BUFFER_LENGTH;i++){
		*codestreamBuffer[i]=&(jpcBuffer[i][0]);
	}
	*bytesInBuffer=&(bytesInJpcBuffer[0]);

}//end GetCodestreamBuffer()

//Copying JPEG2000 codestream data from multriple UDP packets to one array when input video is compressed using one chip.
//Parameters:
//void
//Return: void
void PacketBuffer2Codestream::ClaimFramesSingleChip(){

	debugLog<<"Method not uo to date, GPU procesing not implemented."<<endl;
	exit(-1);

	unsigned int previousSequenceNumber, currentSequenceNumber;
	DWORD waitReturn;
	int currentCodeStream=0;
	bool continuedCodestream=false, currentFieldDone=false;
	int currentPayloadIndex;
	int partitionStart = PAYLOAD_BUFFER_PARTITION_INDEX_0;
	int partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_1;
	int currentPartition = 0;
	
	debugLog<<"Waiting for bufferSemaphore"<<endl;
	waitReturn = WaitForSingleObject(packetBufferSemaphore, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for bufferSemaphore failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}
	debugLog<<"Waiting for bufferMutexPartition0"<<endl;
	waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for bufferMutexPartition0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}

	debugLog<<"Waiting for codestreamBuff0"<<endl;
	waitReturn = WaitForSingleObject(codestreamBuff0, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for codestreamBuff0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}

	//Releasing codestreamBufferSemaphore because access is granted to codestreamBuff0.
	//HostDecoder will wait for this semaphore to be signaled, before it proceed.
	ReleaseSemaphore(codestreamBufferSemaphore, 1, NULL);

	//thread main loop
	while(true){

		//Looping through the partition. A field will be overwritten if the last payload is missing.
		for(currentPayloadIndex = partitionStart; currentPayloadIndex < partitionEnd; currentPayloadIndex++){
			//If the field is divided over two buffer partitions, the copying should be continued without searching for first packet in field.
			if(continuedCodestream){
				goto intermediateProcessing;
			}
			//SOC Offset = 20 byte RTP header + 4 byte T-VIPS hack + 20 MXF UID(Key + length) = 44 byte
			//First video payload in field
			if(((*(updPayloadBuffer[currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[currentPayloadIndex]+21) & 0xff)==0x01)){
				//Copying data
				memcpy(jpcBuffer[currentCodeStream], updPayloadBuffer[currentPayloadIndex]+44, *(bytesInUdpPayload+currentPayloadIndex)-44);
				//Updating the number of bytes in field
				bytesInJpcBuffer[currentCodeStream]=(*(bytesInUdpPayload+currentPayloadIndex)-44);
				//Doing some book keeping
				previousSequenceNumber = ((updPayloadBuffer[currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[currentPayloadIndex][3]&0xFF);
				currentPayloadIndex++;
				continuedCodestream=true;
				currentFieldDone=false;
				
				intermediateProcessing:
				while((!currentFieldDone) && (currentPayloadIndex < partitionEnd)){
					//All intermediate video payloads
					if(((*(updPayloadBuffer[currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[currentPayloadIndex]+21) & 0xff)==0x00)){
						//Checking for sequence number discontinuty
						currentSequenceNumber = ((updPayloadBuffer[currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[currentPayloadIndex][3]&0xFF);
						if(currentSequenceNumber!=(previousSequenceNumber+1)){
							debugLog<<"Sequence number discontinuity."<<endl;
							break;
						}
						previousSequenceNumber = currentSequenceNumber;
						//Copying data
						memcpy(jpcBuffer[currentCodeStream] + bytesInJpcBuffer[currentCodeStream], updPayloadBuffer[currentPayloadIndex]+24, *(bytesInUdpPayload+currentPayloadIndex)-24);
						//Updating the number of bytes in field
						bytesInJpcBuffer[currentCodeStream]+=(*(bytesInUdpPayload+currentPayloadIndex)-24);
						currentPayloadIndex++;
					}
					//Last video payload in field
					else if(((*(updPayloadBuffer[currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[currentPayloadIndex]+21) & 0xff)==0x02)){
						//Checking for sequence number discontinuty
						currentSequenceNumber = ((updPayloadBuffer[currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[currentPayloadIndex][3]&0xFF);
						if(currentSequenceNumber!=(previousSequenceNumber+1)){
							debugLog<<"Sequence number discontinuity."<<endl;
							break;
						}
						//Copying data
						memcpy(jpcBuffer[currentCodeStream] + bytesInJpcBuffer[currentCodeStream], updPayloadBuffer[currentPayloadIndex]+24, *(bytesInUdpPayload+currentPayloadIndex)-24);
						//Updating the number of bytes in field
						bytesInJpcBuffer[currentCodeStream]+=(*(bytesInUdpPayload+currentPayloadIndex)-24);
						//ofstream codeStream;
						//codeStream.open ("dump.jpc", ios::binary );
						//codeStream.write(jpcBuffer[currentCodeStream],bytesInJpcBuffer[currentCodeStream]);
						//codeStream.close();
						currentPayloadIndex++;
						continuedCodestream=false;
						currentFieldDone=true;

						//Gain access to the next codestreamBuff and release the one just updated.
						if(currentCodeStream==0){
							//Must gain access to codestreamBuff1 before codestreamBuff0 is released, this ensure that the decoder "stays in line"
							waitReturn = WaitForSingleObject(codestreamBuff1, 10000);
							if(waitReturn!=WAIT_OBJECT_0) {
								debugLog<<"Wait for codestreamBuff1 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
								if(waitReturn == WAIT_FAILED){
									DWORD lastError = GetLastError();
									debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
								}
								debugLog<<"Exiting.."<<endl;
								exit(-1);
							}
							ReleaseMutex(codestreamBuff0);
							currentCodeStream=1;
						}
						else{
							//Must gain access to codestreamBuff0 before codestreamBuff1 is released, this ensure that the decoder "stays in line"
							waitReturn = WaitForSingleObject(codestreamBuff0, 10000);
							if(waitReturn!=WAIT_OBJECT_0) {
								debugLog<<"Wait for codestreamBuff0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
								if(waitReturn == WAIT_FAILED){
									DWORD lastError = GetLastError();
									debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
								}
								debugLog<<"Exiting.."<<endl;
								exit(-1);
							}
							ReleaseMutex(codestreamBuff1);
							currentCodeStream=0;
						}
					}//Done with last video payload in field
					//Other packet between video payloads: something is wrong, fint start of next field
					else{
						continuedCodestream=false;
						currentFieldDone=true;
					}
				}//Done with field
			}
		}//Done with partition

		//Doing some book keeping, more spesific:
		//Updating loop boundaries, releasing buffer partition mutex and waiting for access to next buffer partition.
		switch(currentPartition){
			case 0x00:
				//debugLog<<"Done with partition 0"<<endl;
				partitionStart = PAYLOAD_BUFFER_PARTITION_INDEX_1;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_2;
				currentPartition = 1;
				waitReturn = WaitForSingleObject(bufferMutexPartition1, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition1 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition0);
				break;

			case 0x01:
				//debugLog<<"Done with partition 1"<<endl;
				partitionStart = PAYLOAD_BUFFER_PARTITION_INDEX_2;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_3;
				currentPartition = 2;
				waitReturn = WaitForSingleObject(bufferMutexPartition2, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition2 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition1);
				break;

			case 0x02:
				//debugLog<<"Done with partition 2"<<endl;
				partitionStart = PAYLOAD_BUFFER_PARTITION_INDEX_3;
				partitionEnd = PAYLOAD_BUFFER_LENGTH;
				currentPartition = 3;
				waitReturn = WaitForSingleObject(bufferMutexPartition3, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition3 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition2);
				break;

			case 0x03:
				//debugLog<<"Done with partition 3"<<endl;
				partitionStart = PAYLOAD_BUFFER_PARTITION_INDEX_0;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_1;
				currentPartition = 0;
				waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition3);
				break;
		}//Next partition determined
	}//end thread main loop
}//end ClaimFramesSingleChip()

//Copying JPEG2000 codestream data from multriple UDP packets to one array when input video is compressed using two chips.
//Parameters:
//void
//Return: void
void PacketBuffer2Codestream::ClaimFramesDualChip(){

	DWORD waitReturn;
	int currentCodeStream=0;
	int currentPayloadIndex = PAYLOAD_BUFFER_PARTITION_INDEX_0;
	int partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_1;
	int currentPartition = 0;
	short currentFrameNumber, previousFrameNumber=0;

	debugLog<<"Waiting for bufferSemaphore"<<endl;
	//This semaphore is signaled once by the socketpacketbuffer class, when access first is given to (packet) buffer partition 0
	//this ensures that the socketpacketbuffer class has started to fill up the packet buffer before this thread try to extract the packets payload from the packet buffer.
	waitReturn = WaitForSingleObject(packetBufferSemaphore, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for bufferSemaphore failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}
	debugLog<<"Waiting for bufferMutexPartition0"<<endl;
	waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for bufferMutexPartition0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}

	debugLog<<"Waiting for codestreamBuff0"<<endl;
	//This semaphore is initialized to two, meaning that two buffers are avalable to be filled with codestream data.
	waitReturn = WaitForSingleObject(codestreamBuff0, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for codestreamBuff0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		debugLog<<"Exiting.."<<endl;
		exit(-1);
	}
	//Releasing codestreamBufferSemaphore because access is granted to codestreamBuff0.
	//HostDecoder will wait for this semaphore to be signaled, before it proceed.
	ReleaseSemaphore(codestreamBufferSemaphore, 1, NULL);
	debugLog<<"Entering thread main loop."<<endl;

	//thread main loop
	while(true){

		//Looping through the partition. A field will be overwritten if the last payload is missing.
		while( currentPayloadIndex < partitionEnd ){
			//If the field is divided over two buffer partitions, the copying should be continued without searching for first packet in field.

			//MXF descriptor indicates the start of a new field
			if((*(updPayloadBuffer[currentPayloadIndex]+20) & 0xFF)==0xFF){

				currentPayloadIndex++;
				//Reconstruct one field ready for decoding.
				ReconstructSingleField(&currentPayloadIndex, &currentCodeStream);
			}
			else{
				currentPayloadIndex++;
			}
		}//Done with partition

		//Doing some book keeping, more spesific:
		//Updating loop boundaries, releasing buffer partition mutex and waiting for access to next buffer partition.
		switch(currentPartition){
			case 0x00:
				//debugLog<<"Done with partition 0"<<endl;
				currentPayloadIndex = PAYLOAD_BUFFER_PARTITION_INDEX_1;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_2;
				currentPartition = 1;
				waitReturn = WaitForSingleObject(bufferMutexPartition1, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition1 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition0);
				break;

			case 0x01:
				//debugLog<<"Done with partition 1"<<endl;
				currentPayloadIndex = PAYLOAD_BUFFER_PARTITION_INDEX_2;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_3;
				currentPartition = 2;
				waitReturn = WaitForSingleObject(bufferMutexPartition2, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition2 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition1);
				break;

			case 0x02:
				//debugLog<<"Done with partition 2"<<endl;
				currentPayloadIndex = PAYLOAD_BUFFER_PARTITION_INDEX_3;
				partitionEnd = PAYLOAD_BUFFER_LENGTH;
				currentPartition = 3;
				waitReturn = WaitForSingleObject(bufferMutexPartition3, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition3 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition2);
				break;

			case 0x03:
				//debugLog<<"Done with partition 3"<<endl;
				currentPayloadIndex = PAYLOAD_BUFFER_PARTITION_INDEX_0;
				partitionEnd = PAYLOAD_BUFFER_PARTITION_INDEX_1;
				currentPartition = 0;
				waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for bufferMutexPartition0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(bufferMutexPartition3);
				break;
		}//Next partition determined
	}//end thread main loop
}//end ClaimFramesDualChip()

//Helper method called from the ClaimFramesDualChip() method.
//Reconstructs a single jpc encoded field from multiple payloads.
//Parameters:
//payloadIndex [in] - Packet number of the first payload.
//currentCodestream [in] - The location number to save the reconstructed codestream.
//Return: Void
void PacketBuffer2Codestream::ReconstructSingleField(int *currentPayloadIndex, int *currentCodeStream){

	bool fieldDone = false;
	int previousSequenceNumber=-1, currentSequenceNumber;
	DWORD waitReturn;

	//Looping through the packets needed to reconstruct field
	while(!fieldDone){

		//First luma payload in field
		if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x01) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x01)){

			//Checking for sequence number discontinuty
			previousSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2] & 0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3] & 0xFF);
			//SOC Offset = 20 byte RTP header + 4 byte T-VIPS hack + 20 MXF UID(Key + length) = 44 byte
			//short socOffset = 44;
			//MarkerSearch(updPayloadBuffer[currentPayloadIndex], *(bytesInUdpPayload+currentPayloadIndex), 0xFF52, true);
			//Copying SOC and SIZ marker segment(45 bytes)
			memcpy(jpcBuffer[*currentCodeStream], updPayloadBuffer[*currentPayloadIndex]+socOffset, 45);
			//Must update the Siz marker segment with info regarding chroma.
			*(jpcBuffer[*currentCodeStream]+5)=0x2F;
			*(jpcBuffer[*currentCodeStream]+41)=0x03;
			*(jpcBuffer[*currentCodeStream]+45)=0x09;
			*(jpcBuffer[*currentCodeStream]+46)=0x02;
			*(jpcBuffer[*currentCodeStream]+47)=0x01;
			*(jpcBuffer[*currentCodeStream]+48)=0x09;
			*(jpcBuffer[*currentCodeStream]+49)=0x02;
			*(jpcBuffer[*currentCodeStream]+50)=0x01;

			//Create Progression Order Change (POC) marker segment
			*(jpcBuffer[*currentCodeStream]+51)=(unsigned)0xFF;//Marker
			*(jpcBuffer[*currentCodeStream]+52)=0x5F;//--||--
			*(jpcBuffer[*currentCodeStream]+53)=0x00;//Lpoc
			*(jpcBuffer[*currentCodeStream]+54)=0x10;//--||--

			*(jpcBuffer[*currentCodeStream]+55)=0x00;//RSpoc
			*(jpcBuffer[*currentCodeStream]+56)=0x00;//CSpoc
			*(jpcBuffer[*currentCodeStream]+57)=0x00;//LYEpoc
			*(jpcBuffer[*currentCodeStream]+58)=0x01;//--||--
			*(jpcBuffer[*currentCodeStream]+59)=0x06;//REpoc
			*(jpcBuffer[*currentCodeStream]+60)=0x01;//CEpoc
			*(jpcBuffer[*currentCodeStream]+61)=0x00;//Ppoc

			*(jpcBuffer[*currentCodeStream]+62)=0x00;//RSpoc
			*(jpcBuffer[*currentCodeStream]+63)=0x01;//CSpoc
			*(jpcBuffer[*currentCodeStream]+64)=0x00;//LYEpoc
			*(jpcBuffer[*currentCodeStream]+65)=0x01;//--||--
			*(jpcBuffer[*currentCodeStream]+66)=0x06;//REpoc
			*(jpcBuffer[*currentCodeStream]+67)=0x03;//CEpoc
			*(jpcBuffer[*currentCodeStream]+68)=0x00;//Ppoc

			//Create Arbitrary Transform Kernel (ATK) marker segment
			//Lossy ATK: FF79 0006 4802 0100
			//Lossless ATK. FF79 0005 5802 00
			*(jpcBuffer[*currentCodeStream]+69)=(unsigned)0xFF;//Marker
			*(jpcBuffer[*currentCodeStream]+70)=0x79;//--||--
			*(jpcBuffer[*currentCodeStream]+71)=0x00;//Latk
			*(jpcBuffer[*currentCodeStream]+72)=0x06;//--||--
			*(jpcBuffer[*currentCodeStream]+73)=0x48;//Satk
			*(jpcBuffer[*currentCodeStream]+74)=0x02;//Satk
			*(jpcBuffer[*currentCodeStream]+75)=0x01;//Katk
			*(jpcBuffer[*currentCodeStream]+76)=0x00;//Natk

			//Copying the remaining data
			memcpy(jpcBuffer[*currentCodeStream]+77, updPayloadBuffer[*currentPayloadIndex]+codOffset, *(bytesInUdpPayload + *currentPayloadIndex)-codOffset);
			//Updating the number of bytes in field: (77 = 45 copied + 6 created in the SIZ marker segment + 18 in POC marker segment + 8 in ATK marker segment)
			bytesInJpcBuffer[*currentCodeStream]=(*(bytesInUdpPayload+*currentPayloadIndex) - codOffset + 77);

			//Changing "Transformation" in COD to use lifting scheme in spesified ATK marker segment. This is done to bypass the IDWT.
			//*(jpcBuffer[*currentCodeStream]+90)=0x02;

			*currentPayloadIndex+=1;
		}//Done with first luma packet

		//First chroma payload in field
		else if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x01)){

			previousSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2] & 0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3] & 0xFF);
			//Searching for SOT marker segment, this is the first marker segment to be copied
			//short sotOffset1 = MarkerSearch(updPayloadBuffer[*currentPayloadIndex], *(bytesInUdpPayload+*currentPayloadIndex), 0xFF90, true);
			//sotOffset1 always 143?
			//Copying data
			memcpy(chromaTempBuffer, updPayloadBuffer[*currentPayloadIndex] + 143, *(bytesInUdpPayload+*currentPayloadIndex) - 143);
			//Updating tile-part index and number of tile-parts for chroma SOT marker segment
			*(chromaTempBuffer+10)=0x01;
			*(chromaTempBuffer+11)=0x02;
			//Updating the number of bytes in field
			bytesInChromaBuffer =(*(bytesInUdpPayload+*currentPayloadIndex)-143);

			*currentPayloadIndex+=1;
		}//Done with first chroma packet

		//Intermediate luma payloads
		else if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x01) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x00)){
			//Checking for sequence number discontinuty
			currentSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3]&0xFF);
			if(currentSequenceNumber!=(previousSequenceNumber+1)){
				//debugLog<<"Sequence number discontinuity, current: "<< currentSequenceNumber <<", previous: "<< previousSequenceNumber <<endl;
				break;
			}
			previousSequenceNumber = currentSequenceNumber;
			//Copying data
			memcpy(jpcBuffer[*currentCodeStream] + bytesInJpcBuffer[*currentCodeStream], updPayloadBuffer[*currentPayloadIndex]+24, *(bytesInUdpPayload+*currentPayloadIndex)-24);
			//Updating the number of bytes in field
			bytesInJpcBuffer[*currentCodeStream]+=(*(bytesInUdpPayload+*currentPayloadIndex)-24);

			*currentPayloadIndex+=1;
		}//Done with intermediate luma packet

		//Intermediate chroma payloads
		else if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x00)){
			//Checking for sequence number discontinuty
			currentSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3]&0xFF);
			if(currentSequenceNumber!=(previousSequenceNumber+1)){
				//debugLog<<"Sequence number discontinuity, current: "<< currentSequenceNumber <<", previous: "<< previousSequenceNumber <<endl;
				break;
			}
			previousSequenceNumber = currentSequenceNumber;
			//Copying data
			memcpy(chromaTempBuffer + bytesInChromaBuffer, updPayloadBuffer[*currentPayloadIndex]+24, *(bytesInUdpPayload+*currentPayloadIndex)-24);
			//Updating the number of bytes in field
			bytesInChromaBuffer+=(*(bytesInUdpPayload+*currentPayloadIndex)-24);

			*currentPayloadIndex+=1;
		}//Done with intermediate chroma packet

		//Last chroma payload in field
		else if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x04) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x02)){
			//Checking for sequence number discontinuty
			currentSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3]&0xFF);
			if(currentSequenceNumber!=(previousSequenceNumber+1)){
				//debugLog<<"Sequence number discontinuity, current: "<< currentSequenceNumber <<", previous: "<< previousSequenceNumber <<endl;
				break;
			}
			previousSequenceNumber = currentSequenceNumber;
			//Copying data
			memcpy(chromaTempBuffer + bytesInChromaBuffer, updPayloadBuffer[*currentPayloadIndex]+24, *(bytesInUdpPayload+*currentPayloadIndex)-24);
			//Updating the number of bytes in field
			bytesInChromaBuffer+=(*(bytesInUdpPayload+*currentPayloadIndex)-24);

			//Removing possible zero padding in luma stream.
			while(*(jpcBuffer[*currentCodeStream]+ bytesInJpcBuffer[*currentCodeStream]-1)==0x00){
				bytesInJpcBuffer[*currentCodeStream]--;
			}
			//Removing EOD from luma stream
			bytesInJpcBuffer[*currentCodeStream]-=2;

			//Finding the offset to start of tail-part marker segment in luma stream
			short sotOffset2 = MarkerSearch(jpcBuffer[*currentCodeStream], *(bytesInJpcBuffer+*currentCodeStream),0xFF90, false);
			//sotOffset always 0x66?

			//Update SOT marker segment for luma
			//Setting the correct number of bytes in luma tile-part (Psot)
			*(jpcBuffer[*currentCodeStream]+sotOffset2+6)= ((*(bytesInJpcBuffer+*currentCodeStream)-sotOffset2) >> 24) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+sotOffset2+7)= ((*(bytesInJpcBuffer+*currentCodeStream)-sotOffset2) >> 16) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+sotOffset2+8)= ((*(bytesInJpcBuffer+*currentCodeStream)-sotOffset2) >> 8) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+sotOffset2+9)= (*(bytesInJpcBuffer+*currentCodeStream)-sotOffset2) & 0xFF;
			//Setting the number of tail-parts (TNsot) to 2
			*(jpcBuffer[*currentCodeStream]+sotOffset2+11)=0x02;

			//Copying chroma data from temporary storage
			memcpy(jpcBuffer[*currentCodeStream] + bytesInJpcBuffer[*currentCodeStream], chromaTempBuffer, bytesInChromaBuffer);
			bytesInJpcBuffer[*currentCodeStream]+=bytesInChromaBuffer;

			//if(true){
			//	ofstream codeStream;
			//	codeStream.open ("dump.jpc", ios::binary );
			//	codeStream.write(jpcBuffer[*currentCodeStream],bytesInJpcBuffer[*currentCodeStream]);
			//	codeStream.close();

			//	codeStream.open ("chroma.tmp", ios::binary );
			//	codeStream.write(chromaTempBuffer,bytesInChromaBuffer);
			//	codeStream.close();

			//	codeStream.open ("packet.pkt", ios::binary );
			//	codeStream.write(updPayloadBuffer[*currentPayloadIndex],bytesInUdpPayload[*currentPayloadIndex]);
			//	codeStream.close();
			//}

			//Gain access to the next codestreamBuff and release the one just updated.
			if(*currentCodeStream==0){
				//Must gain access to codestreamBuff1 before codestreamBuff0 is released, this ensure that the decoder "stays in line"
				waitReturn = WaitForSingleObject(codestreamBuff1, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for codestreamBuff1 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(codestreamBuff0);
				*currentCodeStream=1;
			}
			else{
				//Must gain access to codestreamBuff0 before codestreamBuff1 is released, this ensure that the decoder "stays in line"
				waitReturn = WaitForSingleObject(codestreamBuff0, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for codestreamBuff0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(codestreamBuff1);
				*currentCodeStream=0;
			}

			*currentPayloadIndex+=1;
			fieldDone=true;
		}//Done with last chroma payload

		//Last luma payload in field
		else if(((*(updPayloadBuffer[*currentPayloadIndex]+20) & 0xff)==0x01) && ((*(updPayloadBuffer[*currentPayloadIndex]+21) & 0xff)==0x02)){
			//Checking for sequence number discontinuty
			currentSequenceNumber = ((updPayloadBuffer[*currentPayloadIndex][2]&0xFF) << 8) | (updPayloadBuffer[*currentPayloadIndex][3]&0xFF);
			if(currentSequenceNumber!=(previousSequenceNumber+1)){
				//debugLog<<"Sequence number discontinuity, current: "<< currentSequenceNumber <<", previous: "<< previousSequenceNumber <<endl;
				break;
			}
			//Copying data
			memcpy(jpcBuffer[*currentCodeStream] + bytesInJpcBuffer[*currentCodeStream], updPayloadBuffer[*currentPayloadIndex]+24, *(bytesInUdpPayload+*currentPayloadIndex)-24);
			//Updating the number of bytes in field
			bytesInJpcBuffer[*currentCodeStream]+=(*(bytesInUdpPayload+*currentPayloadIndex)-24);

			//Removing possible zero padding in luma stream.
			while(*(jpcBuffer[*currentCodeStream]+ bytesInJpcBuffer[*currentCodeStream]-1)==0x00){
				bytesInJpcBuffer[*currentCodeStream]--;
			}
			//Removing EOD from luma stream
			bytesInJpcBuffer[*currentCodeStream]-=2;

			//Finding the offset to start of tail-part marker segment in luma stream
			//short sotOffset3 = MarkerSearch(jpcBuffer[*currentCodeStream], *(bytesInJpcBuffer+*currentCodeStream),0xFF90, false);
			//sotOffset always 128?

			//Update SOT marker segment for luma
			//Setting the correct number of bytes in luma tile-part (Psot)
			*(jpcBuffer[*currentCodeStream]+128+6)= ((*(bytesInJpcBuffer+*currentCodeStream)-128) >> 24) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+128+7)= ((*(bytesInJpcBuffer+*currentCodeStream)-128) >> 16) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+128+8)= ((*(bytesInJpcBuffer+*currentCodeStream)-128) >> 8) & 0xFF;
			*(jpcBuffer[*currentCodeStream]+128+9)= (*(bytesInJpcBuffer+*currentCodeStream)-128) & 0xFF;
			//Setting the number of tail-parts (TNsot) to 2
			*(jpcBuffer[*currentCodeStream]+128+11)=0x02;

			//Removing possible zero padding in chroma stream.
			//while(*(chromaTempBuffer + bytesInChromaBuffer-1)==0x00){
			//	bytesInChromaBuffer--;
			//}
			//Copying chroma data from temporary storage
			memcpy(jpcBuffer[*currentCodeStream] + bytesInJpcBuffer[*currentCodeStream], chromaTempBuffer, bytesInChromaBuffer);
			bytesInJpcBuffer[*currentCodeStream]+=bytesInChromaBuffer;

			//if(true){
			//	ofstream codeStream;
			//	codeStream.open ("dump.jpc", ios::binary );
			//	codeStream.write(jpcBuffer[*currentCodeStream],bytesInJpcBuffer[*currentCodeStream]);
			//	codeStream.close();

			//	codeStream.open ("chroma.tmp", ios::binary );
			//	codeStream.write(chromaTempBuffer,bytesInChromaBuffer);
			//	codeStream.close();

			//	codeStream.open ("packet.pkt", ios::binary );
			//	codeStream.write(updPayloadBuffer[*currentPayloadIndex],bytesInUdpPayload[*currentPayloadIndex]);
			//	codeStream.close();
			//}

			//Gain access to the next codestreamBuff and release the one just updated.
			if(*currentCodeStream==0){
				//Must gain access to codestreamBuff1 before codestreamBuff0 is released, this ensure that the decoder "stays in line"
				waitReturn = WaitForSingleObject(codestreamBuff1, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for codestreamBuff1 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(codestreamBuff0);
				*currentCodeStream=1;
			}
			else{
				//Must gain access to codestreamBuff0 before codestreamBuff1 is released, this ensure that the decoder "stays in line"
				waitReturn = WaitForSingleObject(codestreamBuff0, 10000);
				if(waitReturn!=WAIT_OBJECT_0) {
					debugLog<<"Wait for codestreamBuff0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
					if(waitReturn == WAIT_FAILED){
						DWORD lastError = GetLastError();
						debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
					}
					debugLog<<"Exiting.."<<endl;
					exit(-1);
				}
				ReleaseMutex(codestreamBuff1);
				*currentCodeStream=0;
			}
			*currentPayloadIndex+=1;
			fieldDone=true;
		}//Done with last luma packet

		//Other packet between video payloads: something is wrong, fint start of next field
		else{
			//debugLog<<"Unknown packet inserted into field."<<endl;
			*currentPayloadIndex+=1;
			fieldDone=true;
		}
	}//Done with field
}//end ReconstructSingleField()

//Sorts the current partition using insertion sort.
//Not tested or used.
//Parameters:
//partitionNumber [in] - The partition which is to be sorted.
//Return: void
void PacketBuffer2Codestream::SortPartition(int partitionNumber){
	char* tempPayload;
	int* tempBytesInPayload;
	int i, j;
	unsigned short currentSequenceNumber;
	switch(partitionNumber){
		case 0x00:
			for(i=PAYLOAD_BUFFER_PARTITION_INDEX_0+1; i<PAYLOAD_BUFFER_PARTITION_INDEX_1; i++){
				tempPayload=updPayloadBuffer[i];
				tempBytesInPayload=&bytesInUdpPayload[i];
				currentSequenceNumber =  ((updPayloadBuffer[0][2]&0xFF) << 8) | (updPayloadBuffer[0][3]&0xFF);
				j=i-1;
				while((j>=0) && (((((updPayloadBuffer[j][2]&0xFF) << 8) | (updPayloadBuffer[j][3]&0xFF)) > currentSequenceNumber))){
				*updPayloadBuffer[j+1]=*updPayloadBuffer[j];
				bytesInUdpPayload[j+1]=bytesInUdpPayload[j];
				j=j-1;
				}
			*updPayloadBuffer[j+1]=*tempPayload;
			*(bytesInUdpPayload+j+1)=*tempBytesInPayload;
			}
			break;
//insertionSort(array A)
//begin
//    for i := 1 to length[A]-1 do
//    begin
//        value := A[i];
//        j := i-1;
//        while j ? 0 and A[j] > value do
//        begin
//            A[j + 1] := A[j];
//            j := j-1;
//        end;
//        A[j+1] := value;
//    end;
//end;
	}	
}//end SortPartition()

//Fills the structs RtpFixedHeader, RtpPayloadHeader and TvipsHack with data from packet in <bufferNumber>
//Parameters:
//bufferNumber [in] - the number of the buffer containing the packet
//Return: void
void PacketBuffer2Codestream::ParsePacketHeader(int bufferNumber){
	rtpFixedHeader.version = ((updPayloadBuffer[bufferNumber][0] >> 6) & 0x02);
	rtpFixedHeader.padding = ((updPayloadBuffer[bufferNumber][0] >> 5) & 0x01);
	rtpFixedHeader.csrCCount = (updPayloadBuffer[bufferNumber][0] & 0x0F);
	rtpFixedHeader.marker = ((updPayloadBuffer[bufferNumber][1] >> 7) & 0x01);
	rtpFixedHeader.payloadType = (updPayloadBuffer[bufferNumber][1] & 0x7F);
	rtpFixedHeader.sequenceNumber = ((updPayloadBuffer[bufferNumber][12]&0xFF) << 24) | ((updPayloadBuffer[bufferNumber][13]&0xFF) << 16) | ((updPayloadBuffer[bufferNumber][2]&0xFF) << 8) | (updPayloadBuffer[bufferNumber][3]&0xFF);
	rtpFixedHeader.timeStamp = ((updPayloadBuffer[bufferNumber][4]&0xFF) << 24) | ((updPayloadBuffer[bufferNumber][5]&0xFF) << 16) | ((updPayloadBuffer[bufferNumber][6]&0xFF) << 8) | (updPayloadBuffer[bufferNumber][7]&0xFF);
	rtpFixedHeader.ssrc = ((updPayloadBuffer[bufferNumber][8]&0xFF) << 24) | ((updPayloadBuffer[bufferNumber][9]&0xFF) << 16) | ((updPayloadBuffer[bufferNumber][10]&0xFF) << 8) | (updPayloadBuffer[bufferNumber][11]&0xFF);
	
	rtpPayloadHeader.field = ((updPayloadBuffer[bufferNumber][14] >> 7) & 0x01);
	rtpPayloadHeader.klvElement = ((updPayloadBuffer[bufferNumber][14] >> 6) & 0x01);
	rtpPayloadHeader.fieldNumber = (updPayloadBuffer[bufferNumber][15] & 0xFF);
	rtpPayloadHeader.payloadOffset = ((updPayloadBuffer[bufferNumber][16]&0xFF) << 24) | ((updPayloadBuffer[bufferNumber][17]&0xFF) << 16) | ((updPayloadBuffer[bufferNumber][18]&0xFF) << 8) | (updPayloadBuffer[bufferNumber][19]&0xFF);
	
	tvipsHack.fifoIndex = (updPayloadBuffer[bufferNumber][20] & 0xFF);
	tvipsHack.bitsPresent = (updPayloadBuffer[bufferNumber][21] & 0xFF);
	tvipsHack.keyLength = (updPayloadBuffer[bufferNumber][22] & 0xFF);
	tvipsHack.subChannel = (updPayloadBuffer[bufferNumber][23] & 0xFF);
}//end parseHeader()

//Searches through the received packets for a given marker segment.
//Parameters
//
//Return: void
int PacketBuffer2Codestream::MarkerSearch(char * packetData, int packetSize, int targetMarker, bool rtpHeaderPresent){
	int markerOffset=0;
	
	if(rtpHeaderPresent){
		markerOffset = 44;
	}

	//first marker should be SOC.
	int marker = ((*(packetData+markerOffset) << 8) + *(packetData+markerOffset+1)) & 0xFFFF;
	if(marker==0xFF4F){
		markerOffset = markerOffset + 2;
		marker = ((*(packetData+markerOffset) << 8) & 0xFF00) + ((*(packetData+markerOffset+1)) & 0xFF);
	}

	while(marker!=targetMarker){
		markerOffset = markerOffset + 2 + ((*(packetData+markerOffset+2) << 8) + *(packetData+markerOffset+3));
		if(markerOffset<packetSize){
			marker = ((*(packetData+markerOffset) << 8) & 0xFF00) + ((*(packetData+markerOffset+1)) & 0xFF);
		}
		else{
			markerOffset=-1;
			break;
		}
	}
	return markerOffset;
}//markerSearch()

//Iterates through the received packets and writes the Fixed header, payload header and T-VIPS hack for all packets to file.
//Parameters
//
//Return: void
void PacketBuffer2Codestream::PrintPacketHeaders(){

	for(int i = PAYLOAD_BUFFER_PARTITION_INDEX_0; i < PAYLOAD_BUFFER_PARTITION_INDEX_1; i++){
		ParsePacketHeader(i);
		debugLog << "RTP FIXED HEADER for packet number: "<< i << endl;
		debugLog << "RTP version: " << rtpFixedHeader.version<<endl;
		debugLog << "Padding: " << rtpFixedHeader.padding<<endl;
		debugLog << "CSRC count: " << rtpFixedHeader.csrCCount<<endl;
		debugLog << "Marker: " << rtpFixedHeader.marker<<endl;
		debugLog << "Payload type: " << rtpFixedHeader.payloadType<<endl;
		debugLog << "Sequence number : " << rtpFixedHeader.sequenceNumber<<endl;
		debugLog << "Timestamp: " << rtpFixedHeader.timeStamp<<endl;
		debugLog << "SSRC : " << rtpFixedHeader.ssrc<<endl;

		debugLog << "RTP PAYLOAD HEADER"<<endl;
		debugLog << "Field: " << rtpPayloadHeader.field<<endl;
		debugLog << "KLV element : " << rtpPayloadHeader.klvElement<<endl;
		debugLog << "Field number: " << rtpPayloadHeader.fieldNumber<<endl;
		debugLog << "Payload offset : " << rtpPayloadHeader.payloadOffset<<endl;

		debugLog << "T-VIPS HACK"<<endl;
		debugLog << "Fifo Index: " << tvipsHack.fifoIndex<<endl;
		debugLog << "Bits present : " << tvipsHack.bitsPresent<<endl;
		debugLog << "Key length: " << tvipsHack.keyLength<<endl;
		debugLog << "Sub-channel: " << tvipsHack.subChannel<<endl;
		debugLog << endl;
	}
}//end PrintPacketHeaders()

//The main method which starts in a new thread.
//Parameters:
//void
//Return: void
void PacketBuffer2Codestream::Execute(){

	if(inputVideoFormat==SINGLE_CHIP){
		debugLog<<"Entered ClaimFramesSingleChip()"<<endl;
		ClaimFramesSingleChip();
	}
	else if(inputVideoFormat==DUAL_CHIP){
		debugLog<<"Entered ClaimFramesDualChip()"<<endl;
		ClaimFramesDualChip();
	}
}//end Execute()