/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	Class joining a udp multicast and receiving udp packets, i.e. puting them into a packetbuffer.

***********************************************************************************/

#include "Socket2PacketBuffer.h"
#include "CudaLib.h"

//Constructor
//Parameters:
//localIp [out] - Local IP address of local network card.
//Return: 
Socket2PacketBuffer::Socket2PacketBuffer(string *localIp){

	//Initializing debugLog
	remove("socket_debug_log.txt");
	debugLog.open("socket_debug_log.txt",ios_base::out);

	//Allocating memory for payloadBuffer
	for(int i=0; i<PAYLOAD_BUFFER_LENGTH;i++){
		updPayloadBuffer[i]=new (nothrow) char[PAYLOAD_BUFFER_WIDTH];
		//updPayloadBuffer[i]=(char*)AllocateMemHost(PAYLOAD_BUFFER_WIDTH);
		bytesInUdpPayload[i]=0;
		if (updPayloadBuffer[i] == 0) {
			debugLog<<"Allocating  packetbuffer failed in iteration " << i <<endl;
			debugLog<<"Exiting.."<<endl;
			exit(-1);
		}
	}

	//Creating payloadBuffer mutexes and semaphore
	bufferMutexPartition0 = CreateMutex(NULL, false,(LPCTSTR)"bufferMutexPartition0");
	if(bufferMutexPartition0==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating bufferMutexPartition0 with error code 0x"<<hex<<error<<"."<<endl;
	}
	bufferMutexPartition1 = CreateMutex(NULL, false,(LPCTSTR)"bufferMutexPartition1");
	if(bufferMutexPartition1==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating bufferMutexPartition1 with error code 0x"<<hex<<error<<"."<<endl;
	}
	bufferMutexPartition2 = CreateMutex(NULL, false,(LPCTSTR)"bufferMutexPartition2");
	if(bufferMutexPartition2==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating bufferMutexPartition2 with error code 0x"<<hex<<error<<"."<<endl;
	}
	bufferMutexPartition3 = CreateMutex(NULL, false,(LPCTSTR)"bufferMutexPartition3");
	if(bufferMutexPartition3==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating bufferMutexPartition3 with error code 0x"<<hex<<error<<"."<<endl;
	}
	packetBufferSemaphore = CreateSemaphore(NULL, 0,4,(LPCTSTR)"packetBufferSemaphore");
	if(packetBufferSemaphore==NULL){
		DWORD error = GetLastError();
		debugLog<<"Failed creating bufferSemaphore with error code 0x"<<hex<<error<<"."<<endl;
	}

	//Retreving local IP
	WSADATA wsaData;
	//Initializing Winsock
	int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
		if (iResult != 0) {
		debugLog << "WSAStartup failed: " << iResult << endl;
		}
	char ac[80];
    if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) {
		debugLog << "Failed when getting local host name with error code 0d" << WSAGetLastError()<< "." <<endl;
    }

    struct hostent *phe = gethostbyname(ac);
    if (phe == 0) {
        debugLog <<" Bad host lookup." << endl;
    }
	//inet_addr konverterer fra dot separated IP addresse til nettverk byte-order
	struct in_addr addr;
	memcpy(&addr, phe->h_addr_list[0], sizeof(struct in_addr));
	string temp(inet_ntoa(addr));
	*localIp = temp;
}//end Socket2PacketBuffer()

//Initiating socket with UDP IP, UDP port number and local IP.
//Parameters:
//udpPort [in] - port to receive video stream on.
//updIp [in] - IP address to video stream.
//localIp [in] - IP address to local network card.
//Return: void
void Socket2PacketBuffer::Init(int udpPort, string udpIp, string localIp){
	
	bool constructorSucceeded = true;
	//Creating socket
	if ((UDPsocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
		int wsaError = WSAGetLastError();
		constructorSucceeded=false;
		debugLog << "Method socket() failed with error code 0d" << wsaError << "." <<endl;
		}
	BindSocket(udpPort, &constructorSucceeded);
	JoinMulticast(udpIp, localIp, &constructorSucceeded);
	//BindSocket(5500, &constructorSucceeded);
	//JoinMulticast("229.0.0.8", &constructorSucceeded);
	
	if(!constructorSucceeded){
		MessageBox(NULL, (LPCTSTR)"An error occurred when joining multicast", (LPCTSTR)"Socket Error", MB_ICONERROR);
		exit(-1);
		}
}//end Init()

//Binds the port to the socket.
//Parameters:
//multicastPort [in] - The port to bind to.
//succeeded [in] - set to false if multicast join failed.
//Return: void
void Socket2PacketBuffer::BindSocket(unsigned short multicastPort, bool *succeeded){

	client_address.sin_family = AF_INET;
	client_address.sin_port = htons(multicastPort);
	client_address.sin_addr.s_addr = INADDR_ANY;
	//memset(client_addr.sin_zero, '\0', sizeof client_addr.sin_zero);

	TGV430_address.sin_family = AF_INET;
	TGV430_address.sin_port = htons(0); 
	TGV430_address.sin_addr.s_addr = INADDR_ANY;
	//memset(TGV430_adr.sin_zero, '\0', sizeof TGV430_adr.sin_zero);

	if (bind(UDPsocket, (struct sockaddr *)&client_address, sizeof client_address) < 0) {
		int wsaError = WSAGetLastError();
		debugLog<<"Failed in bind(), with error code 0d" << wsaError<<"."<<endl;
		*succeeded = false;
		}
}//end BindSocket()


//Joins a multicast.
//Parameters:
//multicast_IPAddr [in] - The multicast IP address to join, for instance 229.0.0.8
//succeeded [in] - set to false if multicast join failed.
//Return: void
void Socket2PacketBuffer::JoinMulticast(string multicast_IPAddr, string interface_IPAddr, bool *succeeded){

	char ac[80];
    if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) {
		debugLog << "Failed when getting local host name with error code 0d" << WSAGetLastError()<< "."<<endl;
		*succeeded = false;
    }
    debugLog << "Host name is " << ac << "." << endl;

    struct hostent *phe = gethostbyname(ac);
    if (phe == 0) {
        debugLog <<" Bad host lookup." << endl;
		*succeeded = false;
    }
	multicastRequest.imr_multiaddr.s_addr = inet_addr(multicast_IPAddr.c_str());
	//inet_addr konverterer fra dot separated IP addresse til nettverk byte-order
	multicastRequest.imr_interface.s_addr = inet_addr(interface_IPAddr.c_str());

	if (setsockopt(UDPsocket, IPPROTO_IP, IP_ADD_MEMBERSHIP,(char *) &multicastRequest, sizeof(multicastRequest)) < 0) {
		int wsaError = WSAGetLastError();
		debugLog<<"Multicast group join failed (setsockopt()) with error code 0d"<< wsaError<< "."<<endl;
		*succeeded = false;
	  }
}//end JoinMulticast()

//Cleans up the socket in a correct way. 
//Parameters:
//
//Return: void
void Socket2PacketBuffer::CloseSocket(){
	WSACleanup();
}//end close()

//Copy the buffer pointer. 
//Parameters:
//payloadBuffer [in] - Pointer to the packetBuffer pointers.
//bytesInBuffer [in] - Pointer to the packetBuffer size pointers.
//Return: void
void Socket2PacketBuffer::GetPayloadBuffer(char** payloadBuffer[], int** bytesInBuffer){

	for(int i=0;i<PAYLOAD_BUFFER_LENGTH;i++){
		*payloadBuffer[i]=&(updPayloadBuffer[i][0]);
	}
	*bytesInBuffer=&(bytesInUdpPayload[0]);

}//end close()

MxfVideoDescriptor Socket2PacketBuffer::GetVideoDescriptor(INPUT_VIDEO_FORMAT* videoFormat){
	bool foundVideoDescriptor = false;
	int addr_len = sizeof TGV430_address;
	struct MxfVideoDescriptor currentVideoDescriptor;

	struct timeval timeOut;
	timeOut.tv_sec = 1;
	timeOut.tv_usec = 0;
	int selectReturn;

	fd_set readFDS;
	FD_ZERO(&readFDS);


	while(!foundVideoDescriptor){

		FD_SET(UDPsocket, &readFDS);
		//checkin if data is waiting to be read
		selectReturn = select(NULL, &readFDS, NULL, NULL, &timeOut);
		if(selectReturn > 0){
			bytesInUdpPayload[0] = recvfrom(UDPsocket, &updPayloadBuffer[0][0], PAYLOAD_BUFFER_WIDTH , 0,(struct sockaddr *)&TGV430_address, &addr_len);
			//If T-VIPS hack FIFO index = 0xFF: packets contains MXF descriptor..
			if((updPayloadBuffer[0][20] & 0xFF)==0xFF){
				//...but not necessarily a video descriptor, so must check.
				int descriptorOffset = IsVideoDescriptor(&updPayloadBuffer[0][0], bytesInUdpPayload[0]);
				if(descriptorOffset){
					currentVideoDescriptor = ParseVideoDescriptor(&updPayloadBuffer[0][0],descriptorOffset);
					foundVideoDescriptor=true;
				}
			}
		}
		else if(selectReturn==0){
			MessageBox(NULL, (LPCTSTR)"Timeout waiting for video descriptor, check multicast connection.", (LPCTSTR)"Socket Timeout", MB_ICONERROR);
			exit(-1);
		}
		else if(selectReturn==SOCKET_ERROR){
			MessageBox(NULL, (LPCTSTR)"Error waiting for video descriptor, check multicast connection", (LPCTSTR)"Socket Error", MB_ICONERROR);
			exit(-1);
		}
	}
	//Detects whether the received video is encoded in single or dual chip mode.
	int packetsToCheck = 200;
	bool chromaPresent = false, lumaPresent = false;
	for(int index=0; index < packetsToCheck; index++){
		bytesInUdpPayload[index] = recvfrom(UDPsocket, &updPayloadBuffer[index][0], PAYLOAD_BUFFER_WIDTH , 0,(struct sockaddr *)&TGV430_address, &addr_len);
	}
	for(int index=0; index < packetsToCheck; index++){
		//T-VIPS hack for luma packet in field: FIFO-index=0x01
		if((*(updPayloadBuffer[index]+20) & 0xff)==0x01){
			lumaPresent=true;
		}
		//T-VIPS hack for chroma packet in field: FIFO-index=0x04
		if((*(updPayloadBuffer[index]+20) & 0xff)==0x04){
			chromaPresent=true;
		}
	}
	if(lumaPresent && chromaPresent){
		*videoFormat=DUAL_CHIP;
		debugLog << "Video stream is encoded in Dual-chip mode." <<endl;
	}
	else{
		*videoFormat=SINGLE_CHIP;
		debugLog << "Video stream is encoded in Single-chip mode." <<endl;
	}

	return currentVideoDescriptor;
}//end GetVideoDescriptor()

//Checks the packet to see if it containes a MXF video descriptor
//Parameters:
//*packetData [in] - a pointer to the packet that shall be checked
//packetSize [in] - the number og bytes in the packet 
//Return: integer > 0 (index of descriptor start)if packet contains a MXF video descriptor, else returns integer 0.
int Socket2PacketBuffer::IsVideoDescriptor(char* packetData, int packetSize){

	unsigned char mxfVideoDescriptorKey[16];
	mxfVideoDescriptorKey[0]=0x06;
	mxfVideoDescriptorKey[1]=0x0E;
	mxfVideoDescriptorKey[2]=0x2B;
	mxfVideoDescriptorKey[3]=0x34;
	mxfVideoDescriptorKey[4]=0x02;
	mxfVideoDescriptorKey[5]=0x53;
	mxfVideoDescriptorKey[6]=0x01;
	mxfVideoDescriptorKey[7]=0x01;
	mxfVideoDescriptorKey[8]=0x0d;
	mxfVideoDescriptorKey[9]=0x01;
	mxfVideoDescriptorKey[10]=0x01;
	mxfVideoDescriptorKey[11]=0x01;
	mxfVideoDescriptorKey[12]=0x01;
	mxfVideoDescriptorKey[13]=0x01;
	mxfVideoDescriptorKey[14]=0x5A;
	mxfVideoDescriptorKey[15]=0x00;

	//currentDescriptorIndex = 12 byte RTP fixed header + 8 byte RTP payload header + 4 byte T-VIPS hack + 16 byte MXF key + = 24 byte
	int currentIndex=24;
	int keyIndex=currentIndex;
	bool done = false;
	int i;

	while(!done){
		for(i=0;i<16;i++){
			//MXF key currently processed not equal to MXF Video Descriptor key.
			if(*(packetData+currentIndex)!=mxfVideoDescriptorKey[i]){
				//Asuming length field is BER long-form encoded
				currentIndex=keyIndex + 20 + *(packetData+keyIndex+19);
				keyIndex=currentIndex;
				//last MXF processed, video descriptor not found
				if(currentIndex>packetSize){
					currentIndex=0;
					done=true;
				}
				break;
				}
			else{
				currentIndex++;
			}
		}
		if(i==16){
			done = true;
		}
	}
	return currentIndex;
}//end IsVideoDescriptor()

//Fills a MxfVideoDescriptor struct with information from a MXF video descriptor
//Parameters:
//*packetData [in] - a pointer to the packet containing a MXF video descriptor.
//keyOffset [in] - an integer holding the start index of the desriptor
//Return: the descriptor struct
MxfVideoDescriptor Socket2PacketBuffer::ParseVideoDescriptor(char *packetData, int keyOffset){
	struct MxfVideoDescriptor activeVideoDescriptor;
	for(int i = 0; i<8 ;i++){
		activeVideoDescriptor.aspectRatio[i]=0;
		activeVideoDescriptor.sampleRate[i]=0;
		activeVideoDescriptor.pictureEssenceCoding[i]=0;
		activeVideoDescriptor.instanceUID[i]=0;
		activeVideoDescriptor.essenceContainer[i]=0;
	}
	for(int i = 8; i<16 ;i++){
		activeVideoDescriptor.pictureEssenceCoding[i]=0;
		activeVideoDescriptor.instanceUID[i]=0;
		activeVideoDescriptor.essenceContainer[i]=0;
	}
	activeVideoDescriptor.frameLayout=0;
	activeVideoDescriptor.heigth=0;
	activeVideoDescriptor.width=0;
	activeVideoDescriptor.videoLineMap=0;

	bool isNext=true;
	int index = 0;
	//Length field is BER long-form length encoded
	if ((*(packetData+keyOffset) & 0xff)==0x83){
		index = keyOffset+4;
	}
	else
		return activeVideoDescriptor;

	short tag = (*(packetData+index)<<8) | (*(packetData+index+1));
	//debugLog<<hex<<tag<<endl;
	
	while(isNext){

		switch(tag){

			//Instance UID
			case 0x3C0A:
				for(int i=0;i<16;i++){
					activeVideoDescriptor.instanceUID[i]= *(packetData+index+i+4);
				}
				index=index + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"UID"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Sample rate
			case 0x3001:
				for(int i=0;i<8;i++){
					activeVideoDescriptor.sampleRate[i]= *(packetData+index+i+4);
				}
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"sample rate"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;
			
			//Essence container
			case 0x3004:
				for(int i=0;i<16;i++){
					activeVideoDescriptor.essenceContainer[i]= *(packetData+index+i+4);
				}
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"essence container"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Frame layout
			case 0x320C:
				activeVideoDescriptor.frameLayout= *(packetData+index+4);
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"frame layout"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Heigth
			case 0x3202:
				activeVideoDescriptor.heigth= ((*(packetData+index+6)<<8)&0xFF00) | (*(packetData+index+7)&0xFF);
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"heigth"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Width
			case 0x3203:
				activeVideoDescriptor.width= ((*(packetData+index+6)<<8)&0xFF00) | (*(packetData+index+7)&0xFF);
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"width"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Aspect ration
			case 0x320E:
				for(int i=0;i<8;i++){
					activeVideoDescriptor.aspectRatio[i]= *(packetData+index+i+4);
				}
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"aspect ratio"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Video Line Map
			case 0x320D:
				activeVideoDescriptor.videoLineMap= *(packetData+index+4);
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"video line map"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Picture Essence Coding
			case 0x3201:
				for(int i=0;i<16;i++){
					activeVideoDescriptor.pictureEssenceCoding[i]= *(packetData+index+i+4);
				}
				index=index + (*(packetData+index+2)<<8) + *(packetData+index+3) + 4;
				tag = (*(packetData+index)<<8) | (*(packetData+index+1));
				//debugLog<<"picture essence coding"<<endl;
				//debugLog<<hex<<index<<" "<<tag<<endl;
				break;

			//Active Format Descriptor
			case 0x3218:
				activeVideoDescriptor.activeFormatDescriptor=*(packetData+index+4);
				//debugLog<<"Active Format Descriptor"<<endl;
				isNext=false;
			
			default:
				isNext=false;
		}
	}
	return activeVideoDescriptor;
}//end ParseVideoDescriptor()

//Prints a MxfVideoDescriptor struct to debug log.
//Parameters:
//*videoDescriptor [in] - a pointer to the struct containing the MXF video descriptor.
//Return: void
void Socket2PacketBuffer::PrintVideoDescriptor(struct MxfVideoDescriptor* videoDescriptor){
	debugLog<<"MXF Video Descriptor:"<<endl;

	debugLog<<"Instance UID: 0x";
	for(int i=0;i<15;i++){
			debugLog<<hex<<((*videoDescriptor).instanceUID[i] & 0xFF)<<".";
	}
	debugLog<<((*videoDescriptor).instanceUID[15]&0xFF)<<endl;

	debugLog<<"Heigth: " <<dec<<((*videoDescriptor).heigth)<<endl;
	debugLog<<"Width: "<<dec<<(*videoDescriptor).width<<endl;

	debugLog<<"Frame layput: "<<(*videoDescriptor).frameLayout;
	if((*videoDescriptor).frameLayout==0x00){
		debugLog<<" (Full frame)"<<endl;
	}
	else if((*videoDescriptor).frameLayout==0x01){
		debugLog<<" (Separate fields)"<<endl;
	}
	else if((*videoDescriptor).frameLayout==0x02){
		debugLog<<" (Single field)"<<endl;
	}
	else if((*videoDescriptor).frameLayout==0x03){
		debugLog<<" (Mixed field)"<<endl;
	}
	else if((*videoDescriptor).frameLayout==0x04){
		debugLog<<" (Segmented frame)"<<endl;
	}
	else
		debugLog<<endl;


	debugLog<<"Video Line Map: "<<(*videoDescriptor).videoLineMap<<endl;

	debugLog<<"Essence container: 0x";
	for(int i=0;i<15;i++){
		debugLog<<hex<<((*videoDescriptor).essenceContainer[i] & 0xFF)<<".";
	}
	debugLog<<((*videoDescriptor).essenceContainer[15]&0xFF)<<endl;

	debugLog<<"Picture Essence Coding: 0x";
	for(int i=0;i<15;i++){
		debugLog<<hex<<((*videoDescriptor).pictureEssenceCoding[i]&0xFF)<<".";
	}
	debugLog<<((*videoDescriptor).pictureEssenceCoding[15]&0xFF)<<endl;

	debugLog<<"Aspect ratio: ";
	debugLog<<dec<<((*videoDescriptor).aspectRatio[3]&0xFF)<<":"<<((*videoDescriptor).aspectRatio[7]&0xFF)<<endl;

	debugLog<<"Sample rate: ";
	debugLog<<dec<<(((*videoDescriptor).sampleRate[3]&0xFF)/((*videoDescriptor).sampleRate[7]&0xFF))<<" Hz."<<endl;

	debugLog<<"Active Format Descriptor: "<< ((*videoDescriptor).activeFormatDescriptor&0xFF)<<endl;

	debugLog<<endl;
}//end PrintVideoDescriptor()

//The main method which starts in a new thread
//Parameters:
//void
//Return: void
void Socket2PacketBuffer::Execute(){
	int index;
	DWORD waitReturn;
	int addr_len = sizeof TGV430_address;

	waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
	if(waitReturn!=WAIT_OBJECT_0) {
		debugLog<<"Wait for bufferMutexPartition0 failed with error code 0x"<<hex<<waitReturn<<"."<<endl;
		if(waitReturn == WAIT_FAILED){
			DWORD lastError = GetLastError();
			debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
		}
		CloseSocket();
		exit(-1);
	}
	//Releasing bufferSemaphore because access is granted to bufferMutexPartition0.
	//Packetbuffer2Codestream will wait for this semaphore to be signaled, before it proceed.
	ReleaseSemaphore(packetBufferSemaphore, 1, NULL);
	
	//thread main loop
	while(true){

		////Starting with partition0
		//debugLog<<"Start partition 0"<<endl;
		for(index = PAYLOAD_BUFFER_PARTITION_INDEX_0; index < PAYLOAD_BUFFER_PARTITION_INDEX_1; index++){
			//receive packet
			bytesInUdpPayload[index] = recvfrom(UDPsocket, &updPayloadBuffer[index][0], PAYLOAD_BUFFER_WIDTH , 0, (struct sockaddr *)&TGV430_address, &addr_len);
		}
		//Error check for last received packet
		if(bytesInUdpPayload[index-1]<0){
			int wsaError = WSAGetLastError();
			debugLog<< "Error in packet receiving in partition 0(recvfrom() method) with error code 0x"<<hex<<wsaError<<"."<<endl;
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}

		waitReturn = WaitForSingleObject(bufferMutexPartition1, 10000);
		if(waitReturn!=WAIT_OBJECT_0) {
			debugLog<<"Wait for bufferMutexPartition1 failed in main loop with error code 0x"<<hex<<waitReturn<<"."<<endl;
			if(waitReturn == WAIT_FAILED){
				DWORD lastError = GetLastError();
				debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
			}
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}
		ReleaseMutex(bufferMutexPartition0);
		////end partition0
		////Starting with partition1
		//debugLog<<"Done partition 0"<<endl;
		//debugLog<<"Start partition 1"<<endl;
		for(index = PAYLOAD_BUFFER_PARTITION_INDEX_1; index < PAYLOAD_BUFFER_PARTITION_INDEX_2; index++){
			//receive packet
			bytesInUdpPayload[index] = recvfrom(UDPsocket, &updPayloadBuffer[index][0], PAYLOAD_BUFFER_WIDTH , 0,(struct sockaddr *)&TGV430_address, &addr_len);
		}
		//Error check for last received packet
		if(bytesInUdpPayload[index-1]<0){
			int wsaError = WSAGetLastError();
			debugLog<< "Error in packet receiving in partition 1 (recvfrom() method) with error code 0x"<<hex<<wsaError<<"."<<endl;
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}

		waitReturn = WaitForSingleObject(bufferMutexPartition2, 10000);
		if(waitReturn!=WAIT_OBJECT_0) {
			debugLog<<"Wait for bufferMutexPartition2 failed in main loop with error code 0x"<<hex<<waitReturn<<"."<<endl;
			if(waitReturn == WAIT_FAILED){
				DWORD lastError = GetLastError();
				debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
			}
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}
		ReleaseMutex(bufferMutexPartition1);
		////end partition1
		////Starting with partition2
		//debugLog<<"Done partition 1"<<endl;
		//debugLog<<"Start partition 2"<<endl;
		for(index = PAYLOAD_BUFFER_PARTITION_INDEX_2; index < PAYLOAD_BUFFER_PARTITION_INDEX_3; index++){
			//receive packet
			bytesInUdpPayload[index] = recvfrom(UDPsocket, &updPayloadBuffer[index][0], PAYLOAD_BUFFER_WIDTH , 0,(struct sockaddr *)&TGV430_address, &addr_len);
		}
		//Error check for last received packet
		if(bytesInUdpPayload[index-1]<0){
			int wsaError = WSAGetLastError();
			DWORD lastError = GetLastError();
			debugLog<< "Error in packet receiving in partition 2 (recvfrom() method) with error code 0x"<<hex<<wsaError<<"."<<endl;
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}

		waitReturn = WaitForSingleObject(bufferMutexPartition3, 10000);
		if(waitReturn!=WAIT_OBJECT_0) {
			debugLog<<"Wait for bufferMutexPartition3 failed in main loop with error code 0x"<<hex<<waitReturn<<"."<<endl;
			if(waitReturn == WAIT_FAILED){
				DWORD lastError = GetLastError();
				debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
			}
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}
		ReleaseMutex(bufferMutexPartition2);
		////end partition2
		////Starting with partition3
		//debugLog<<"Done partition 2"<<endl;
		//debugLog<<"Start partition 3"<<endl;
		for(index = PAYLOAD_BUFFER_PARTITION_INDEX_3; index < PAYLOAD_BUFFER_LENGTH; index++){
			//receive packet
			bytesInUdpPayload[index] = recvfrom(UDPsocket, &updPayloadBuffer[index][0], PAYLOAD_BUFFER_WIDTH , 0,(struct sockaddr *)&TGV430_address, &addr_len);
		}
		//Error check for last received packet
		if(bytesInUdpPayload[index-1]<0){
			int wsaError = WSAGetLastError();
			debugLog<< "Error in packet receiving in partition 3 (recvfrom() method) with error code 0x"<<hex<<wsaError<<"."<<endl;
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}

		waitReturn = WaitForSingleObject(bufferMutexPartition0, 10000);
		if(waitReturn!=WAIT_OBJECT_0) {
			debugLog<<"Wait for bufferMutexPartition1 failed in main loop with error code 0x"<<hex<<waitReturn<<"."<<endl;
			if(waitReturn == WAIT_FAILED){
				DWORD lastError = GetLastError();
				debugLog<<"GetLastError() returned: 0x"<<hex<<lastError<<endl;
			}
			debugLog<<"Exiting..." << endl;
			CloseSocket();
			exit(-1);
		}
		ReleaseMutex(bufferMutexPartition3);
		////end partition3
		//debugLog<<"Done partition 3"<<endl;
		//break;
	}//end of thread main loop

}//end Execute()