/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	Class joining a udp multicast and receiving udp packets, i.e. puting them into a packetbuffer.

***********************************************************************************/
#ifndef SOCKET2PACKETBUFFER_H
#define SOCKET2PACKETBUFFER_H

#include <winsock2.h>
#include <ws2tcpip.h>
#include "Thread.h"
#include <fstream>

using namespace std;

enum INPUT_VIDEO_FORMAT{
	SINGLE_CHIP,	//one chip encoding
	DUAL_CHIP	//dual chip encoding
};

struct MxfVideoDescriptor{
	unsigned char instanceUID[16];
	unsigned char essenceContainer[16];
	unsigned char sampleRate[8];
	unsigned char aspectRatio[8];
	int videoLineMap;
	unsigned short heigth, width;
	unsigned char pictureEssenceCoding[16];
	unsigned short frameLayout;
	unsigned short activeFormatDescriptor;
};

//MTU for ethernet 1.5k bytes
//Max UDP packet size is 65 507 bytes.
#define PAYLOAD_BUFFER_WIDTH 1100
#define PAYLOAD_BUFFER_LENGTH 16384//tested with 1024-16384, should be small to obtain low latency, high bit rates get higher frame rate with large buffer.
#define PAYLOAD_BUFFER_PARTITION_INDEX_0 0
#define PAYLOAD_BUFFER_PARTITION_INDEX_1 PAYLOAD_BUFFER_LENGTH/4
#define PAYLOAD_BUFFER_PARTITION_INDEX_2 PAYLOAD_BUFFER_LENGTH/2
#define PAYLOAD_BUFFER_PARTITION_INDEX_3 PAYLOAD_BUFFER_LENGTH*3/4

class Socket2PacketBuffer:public Thread{

public:
	Socket2PacketBuffer(string *localIp);
	//~Socket2PacketBuffer();
	void CloseSocket();
	void Execute();
	void Init(int udpPort, string udpIp, string localIp);
	void GetPayloadBuffer(char** payloadBuffer[], int** bytesInBuffer);
	MxfVideoDescriptor GetVideoDescriptor(INPUT_VIDEO_FORMAT* videoFormat);
	void PrintVideoDescriptor(struct MxfVideoDescriptor* videoDescriptor);

private:
	void BindSocket(unsigned short multicastPort, bool *succeeded);
	void JoinMulticast(string multicast_IPAddr, string interface_IPAddr, bool *succeeded);
	int IsVideoDescriptor(char* packetData, int packetSize);
	MxfVideoDescriptor ParseVideoDescriptor(char *packetData, int descriptorOffset);

	
	int UDPsocket;
	char* updPayloadBuffer[PAYLOAD_BUFFER_LENGTH];
	int bytesInUdpPayload[PAYLOAD_BUFFER_LENGTH];
	struct sockaddr_in client_address;
	struct sockaddr_in TGV430_address;
	struct ip_mreq multicastRequest;
	ofstream debugLog;
	HANDLE bufferMutexPartition0,bufferMutexPartition1,bufferMutexPartition2,bufferMutexPartition3;
	HANDLE packetBufferSemaphore;
};

#endif