/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	Class which display a startup wizard and therby obtaining the UDP address, UDP port and local IP.

***********************************************************************************/

#include "StartupWizard.h"

//Contructor
//Parameters:
//argc [in] - Number of commandline parameters, set to zero.
//argv [in] - The actual commandline parameters.
//portNumber [out] - The upd multicast port obtained from the wizard.
//ip [out] - The upd multicast IP obtained from the wizard.
//localIp [out] - The local network card IP obtained from the wizard.
//Return:
StartupWizard::StartupWizard(int argc, char *argv[], int* portNumber, std::string* ip, std::string* localIp){

	QApplication app(argc, argv);
	QString translatorFileName = QLatin1String("qt_");
	translatorFileName += QLocale::system().name();
	QTranslator *translator = new QTranslator(&app);
	if (translator->load(translatorFileName, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
	app.installTranslator(translator);

	QWizard wizard;
	StartupWizardPage *startupWizardPage = new StartupWizardPage(portNumber, ip, localIp);
	wizard.addPage((QWizardPage*)startupWizardPage);

	wizard.setWindowTitle("JPEG2000 Video Decoder - Startup Wizard");
	wizard.setOption(QWizard::NoBackButtonOnLastPage, true);
	wizard.show();

	app.exec();
	*ip = wizard.field("ipLineEdit").toByteArray();
	*portNumber = wizard.field("portLineEdit").toInt();
	*localIp = wizard.field("localIpLineEdit").toByteArray();
}// StartupWizard()

//Constructor
//Parameters:
//portNumber [out] - The upd multicast port obtained from the wizard.
//ip [out] - The upd multicast IP obtained from the wizard.
//Return:
StartupWizardPage::StartupWizardPage(int* portNumber, std::string* ip, std::string* localIp){

	setTitle("Fill in IP address and port number for the multicast.");
	setSubTitle("Local IP only need if the computer has multiple network cards.");

	setButtonText( QWizard::FinishButton,"Start decoder");
	setButtonText( QWizard::CancelButton,"Exit");

	QLabel *ipLabel = new QLabel("IP:");
	ipLineEdit = new QLineEdit;
	registerField("ipLineEdit", ipLineEdit);

	QLabel *portLabel = new QLabel("Port:");
	portLineEdit = new QLineEdit;
	registerField("portLineEdit", portLineEdit);

	QLabel *localIpLabel = new QLabel("Local IP:");
	localIpLineEdit = new QLineEdit;
	registerField("localIpLineEdit", localIpLineEdit);

	if(*portNumber!=0){
		ipLineEdit->setText(QString::fromStdString(*ip));
		QString temp;
		portLineEdit->setText(temp.setNum(*portNumber));
		localIpLineEdit->setText(QString::fromStdString(*localIp));
		ipLineEdit->home(true);
	}
	else
		localIpLineEdit->setText("0.0.0.0");

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(ipLabel, 0, 0);
	layout->addWidget(ipLineEdit, 0, 1);
	layout->addWidget(portLabel, 1, 0);
	layout->addWidget(portLineEdit, 1, 1);
	layout->addWidget(localIpLabel, 2, 0);
	layout->addWidget(localIpLineEdit, 2, 1);
	setLayout(layout);
}//end StartupWizardPage()