/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	JPEG2000 code stream source, inherits from kdu_compressed_source.

***********************************************************************************/

#include "kduCompressedMemorySource.h"
#include "CUdaLIb.h"

kduCompressedMemorySource::kduCompressedMemorySource(){
	
}//


//Constructor used to create jpc memory source from a file.
void kduCompressedMemorySource::init(const char *fname){

	currentOffset = 0;
	bytesInJpcBufferP = &bytesInJpcBuffer;
	capabilities =  KDU_SOURCE_CAP_IN_MEMORY | KDU_SOURCE_CAP_SEEKABLE | KDU_SOURCE_CAP_SEQUENTIAL;

	FILE * inputFile;
	inputFile = fopen (fname , "rb");
	if (inputFile==NULL){
		fputs ("File error",stderr);
		exit (EXIT_FAILURE);
	}

	// obtain file size:
	fseek (inputFile , 0 , SEEK_END);
	*bytesInJpcBufferP = ftell (inputFile);
	rewind (inputFile);

	// allocate memory to contain the whole file:
	jpcBuffer = (kdu_byte*) malloc (sizeof(char)*1250000);
	if (jpcBuffer == NULL){
		fputs ("Memory allocation error",stderr); 
		exit (EXIT_FAILURE);
	}
	// copy the file into buffer:
	size_t result = fread (jpcBuffer,1,*bytesInJpcBufferP,inputFile);
	if (result != *bytesInJpcBufferP){
		fputs ("Reading from file error",stderr); 
		exit (EXIT_FAILURE);
	}
}


//Constructor used to create jpc memory source from a given memory buffer.
void kduCompressedMemorySource::init(){

	currentOffset = 0;
	capabilities =  KDU_SOURCE_CAP_IN_MEMORY | KDU_SOURCE_CAP_SEEKABLE | KDU_SOURCE_CAP_SEQUENTIAL;

	jpcBuffer = (kdu_byte*) malloc (sizeof(char)*1250000);
	if (jpcBuffer == NULL){
		fputs ("Memory allocation error",stderr); 
		exit (EXIT_FAILURE);
	}

	//jpcBuffer=(kdu_byte *)*codestreamBuffer;
	//bytesInJpcBufferP = (unsigned int*)*bytesInBuffer;
}

//load codestream data into memory source
void kduCompressedMemorySource::update(char* codestreamData, unsigned int bytesInCodestreamData){

	//memcpy(jpcBuffer, codestreamData, bytesInCodestreamData);
	jpcBuffer=(kdu_byte*)codestreamData;
	bytesInJpcBuffer = bytesInCodestreamData;

}

int kduCompressedMemorySource::get_capabilities(){

	return capabilities;

}

kdu_long kduCompressedMemorySource::get_pos(){

	return currentOffset;
}

kdu_byte* kduCompressedMemorySource::access_memory(kdu_long &pos, kdu_byte * &lim){
	pos=currentOffset;
	lim=jpcBuffer + *bytesInJpcBufferP;
	return jpcBuffer+currentOffset;

}

int kduCompressedMemorySource::read(kdu_byte *buf, int num_bytes){
	if((currentOffset+num_bytes) < *bytesInJpcBufferP){
		memcpy(buf+currentOffset, jpcBuffer,num_bytes);
		currentOffset+=num_bytes;
		return num_bytes;
	}
	else{
		memcpy(buf+currentOffset, jpcBuffer,*bytesInJpcBufferP - currentOffset-1);
		currentOffset= *bytesInJpcBufferP - 1;
		return *bytesInJpcBufferP - currentOffset-1;
	}
}

bool kduCompressedMemorySource::seek(kdu_long offset){

	currentOffset += offset;

	return true;
}
bool kduCompressedMemorySource::close(){

	return true;
}

void kduCompressedMemorySource::saveBufferContentToFile(){

	std::ofstream codeStream;
	codeStream.open ("decoderInput.jpc", std::ios::binary );
	codeStream.write((const char*)jpcBuffer,*bytesInJpcBufferP);
	codeStream.close();

}
