/**********************************************************************************

	AUTHOR: Erlend Halsteinli
	CREATED: SPRING 2009 @ IET NTNU

	JPEG2000 Video Decoder Application, designed to decode a 720p JPEG2000 codestream at 50fps.

	Consists of five modules:

	-Input of UPD address and port number, defined in class StartupWizard.
	-Receive UPD packets, one thread, defined in class Socket2PacketBuffer.
	-Reconstruct compressed frame from multiple payloads, one thread, defined in class PacketBuffer2Codestream.
	-Decoding done on CPU, two threads, defined in class Jpeg2000Decoder.
	-Decoding done on GPU and GUI, three threads, defined in the library CudaLib.

***********************************************************************************/

#include "Socket2PacketBuffer.h"
#include "PacketBuffer2CodeStream.h"
#include "StartupWizard.h"
#include "HostDecoder.h"
#include <CudaLib.h>
#include <time.h>

//int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
int main(int argc, char *argv[]){

	remove("main_debug_log.txt");
	ofstream debugLog;
	debugLog.open("main_debug_log.txt",ios_base::out);

	int udpPort=5500;
	std::string udpIp("229.0.0.8"), localIp("129.241.3.135");
	//int argc=0;
	//char* argv[1];
	//Creating socket to get local IP
	Socket2PacketBuffer socket2Buffer(&localIp);
	//Starting wizard to get UDP IP, UDP port number and local IP.
	StartupWizard startupWizard(argc,argv,&udpPort,&udpIp,&localIp);
	//Initialing socket with information from wizard
	socket2Buffer.Init(udpPort, udpIp, localIp);

	int** bytesInBuffer=new int*;
	char **packetBuffer[PAYLOAD_BUFFER_LENGTH];
	for(int i=0;i<PAYLOAD_BUFFER_LENGTH;i++){
		packetBuffer[i]=new char*;
	}
	enum INPUT_VIDEO_FORMAT videoFormat;
	socket2Buffer.GetPayloadBuffer(packetBuffer, bytesInBuffer);
	//Obtaining and printing video descriptor
	struct MxfVideoDescriptor currentVideoDescriptor = socket2Buffer.GetVideoDescriptor(&videoFormat);
	socket2Buffer.PrintVideoDescriptor(&currentVideoDescriptor);

	//Creating PacketBuffer2Codestream object
	char **codestreamBuffer[2];
	for(int i=0;i<2;i++){
		codestreamBuffer[i]=new char*;
	}
	PacketBuffer2Codestream packet2Stream(packetBuffer, bytesInBuffer, videoFormat);
	packet2Stream.GetCodestreamBuffer(codestreamBuffer, bytesInBuffer);
	
	//Creating decoder object
	HostDecoder hostDecoder(codestreamBuffer, bytesInBuffer);
	unsigned int buffHeigth = NULL;
	if(argc>1){
		buffHeigth= (unsigned int)*(argv[1]);
		if (buffHeigth==0x30)
			buffHeigth=NULL;
			else if(buffHeigth==0x31)
				buffHeigth=2;
			else if(buffHeigth==0x32)
				buffHeigth=4;
			else if(buffHeigth==0x33)
				buffHeigth=8;
			else if(buffHeigth==0x34)
				buffHeigth=16;
			else if(buffHeigth==0x35)
				buffHeigth=32;
			else if(buffHeigth==0x36)
				buffHeigth=64;
			else
				buffHeigth=NULL;
	}

	fprintf(stderr, "buffHeigth: %i \n",buffHeigth);

	////Testing both Kakadu and GPU decoding
	//Sleep(10);
	//int  loopCount=2000;
	//clock_t startClk, endClk;
	//float loopTime;

	//startClk = clock();
	//for(int i=0; i<loopCount;i++){
	//	PerformanceTest();	
	//}
	//endClk=clock();
	//loopTime = ((float)(endClk-startClk))/CLOCKS_PER_SEC;
	//fprintf(stderr, "Complete decoding loop done, decoded %i frames in %f seconds\n",loopCount, loopTime);
	//fprintf(stderr, "fps: %f \n",((float)loopCount)/loopTime);
	//return 0;

	//Forking new thread
	HANDLE socketH = socket2Buffer.Start();
	
	//Forking new thread
	HANDLE streamH = packet2Stream.Start();

	hostDecoder.SetDbufHeight(buffHeigth);
	//Forking new thread
	HANDLE decoderH = hostDecoder.Start();

	Sleep(100);
	StartGuiMainloop();

	debugLog<<"Main thread exiting..."<<endl;
    return 0;
}